@@ -93,13 +93,9 @@ module Merb
         until subclasses.empty?
           time = Time.now.to_i
           bootloader = subclasses.shift
-          if (ENV['DEBUG'] || $DEBUG || Merb::Config[:verbose]) && Merb.logger
-            Merb.logger.debug!("Loading: #{bootloader}")
-          end
+          Merb.logger.debug!("Loading: #{bootloader}") if Merb.verbose_logging?
           Object.full_const_get(bootloader).run
-          if (ENV['DEBUG'] || $DEBUG || Merb::Config[:verbose]) && Merb.logger
-            Merb.logger.debug!("It took: #{Time.now.to_i - time}")
-          end
+          Merb.logger.debug!("It took: #{Time.now.to_i - time}") if Merb.verbose_logging?
           self.finished << bootloader
         end
         self.subclasses = subklasses
@@ -221,24 +217,8 @@ class Merb::BootLoader::Logger < Merb::BootLoader
 
     Merb::Config[:log_stream] = 
       Merb::Config[:original_log_stream] || Merb.log_stream
-
-    print_warnings
-
     nil
   end
-
-  # Print a warning if the installed version of rubygems is not supported
-  #
-  # ==== Returns
-  # nil
-  #
-  # :api: private
-  def self.print_warnings
-    if Gem::Version.new(Gem::RubyGemsVersion) < Gem::Version.new("1.1")
-      Merb.fatal! "Merb requires Rubygems 1.1 and later. " \
-        "Please upgrade RubyGems with gem update --system."
-    end
-  end
 end
 
 # Stores pid file.
@@ -325,6 +305,7 @@ class Merb::BootLoader::BuildFramework < Merb::BootLoader
     # :api: plugin
     def run
       $:.push Merb.root unless Merb.root == File.expand_path(Dir.pwd)
+      STDOUT.puts "Merb root at: #{Merb.root}" unless Merb.testing?
       build_framework
       nil
     end
@@ -357,19 +338,29 @@ class Merb::BootLoader::BuildFramework < Merb::BootLoader
 end
 
 class Merb::BootLoader::Dependencies < Merb::BootLoader
-
   # ==== Returns
   # Array[Gem::Dependency]:: The dependencies registered in init.rb.
   #
+  # As of Merb.1.1 these dependencies only get loaded when
+  # Merb::Config[:kernel_dependencies] is set to true. 
+  # 
+  # This will be removed as of Merb 2.0
+  # 
   # :api: plugin
+  # @deprecated
   cattr_accessor :dependencies
   self.dependencies = []
 
   # Load the init_file specified in Merb::Config or if not specified, the
   # init.rb file from the Merb configuration directory, and any environment
-  # files, which register the list of necessary dependencies and any
-  # after_app_loads hooks.
+  # files and any after_app_loads hooks.
   #
+  # Dependencies are loaded via Bunlder and managed in the Gemfile manifest.
+  # By default manifest for Bundler is in the root directory of the app and
+  # is called Gemfile. All dependencies MUST be definied there because all
+  # dependency hangling was removed from Merb.
+  #
+  # ==== Deprecated (1.0.x)
   # Dependencies can hook into the bootloader process itself by using
   # before or after insertion methods. Since these are loaded from this
   # bootloader (Dependencies), they can only adapt the bootloaders that
@@ -381,44 +372,52 @@ class Merb::BootLoader::Dependencies < Merb::BootLoader
   # :api: plugin
   def self.run
     set_encoding
-    # this is crucial: load init file with all the preferences
-    # then environment init file, then start enabling specific
-    # components, load dependencies and update logger.
+    load_dependencies
     unless Merb::disabled?(:initfile)
       load_initfile
       load_env_config
     end
     expand_ruby_path
     enable_json_gem unless Merb::disabled?(:json)
-    load_dependencies
     update_logger
     nil
   end
-
-  # Load each dependency that has been declared so far.
+  
+  # Try to load the gem environment file (set via Merb::Config[:gemenv])
+  # defaults to ./gems/environment
   #
+  # Load each the dependencies defined in the Merb::Config[:gemfile] 
+  # using the bundler gem's Bundler::require_env
+  # 
+  # Falls back to rubygems if no bundler environment exists
+  # 
   # ==== Returns
   # nil
   #
   # :api: private
   def self.load_dependencies
-    dependencies.each { |dependency| Kernel.load_dependency(dependency, nil) }
+    begin
+      Bundler.require(:default, Merb.environment.to_sym)
+    rescue Bundler::GemfileNotFound
+      Merb.logger.error! "No Gemfile found! If you're generating new app with merb-gen " \
+                         "this is fine, otherwise run: bundle init to create Gemfile"
+    end
     nil
   end
-
-  # Loads json or json_pure and requires it.
+  
+  # Requires json or json_pure.
   #
   # ==== Returns
   # nil
   #
   # :api: private
   def self.enable_json_gem
-    gem "json"
-    require "json/ext"
-  rescue LoadError
-    gem "json_pure"
-    require "json/pure"
-    require "merb-core/core_ext/json_pure_fix"
+    require "json"
+    rescue LoadError
+        Merb.logger.error! "You have enabled JSON but don't have json " \
+                           "installed or don't have dependency in the Gemfile. " \
+                           "Add \"gem 'json', '>= 1.1.7'\" or " \
+                           "\"gem 'json_pure', '>= 1.1.7'\" to your Gemfile."
   end
 
   # Resets the logger and sets the log_stream to Merb::Config[:log_file]
@@ -433,9 +432,15 @@ class Merb::BootLoader::Dependencies < Merb::BootLoader
 
     # If log file is given, use it and not log stream we have.
     if Merb::Config[:log_file]
-      raise "log file should be a string, got: #{Merb::Config[:log_file].inspect}" unless Merb::Config[:log_file].is_a?(String)
-      STDOUT.puts "Logging to file at #{Merb::Config[:log_file]}" unless Merb.testing?
-      Merb::Config[:log_stream] = File.open(Merb::Config[:log_file], "a")
+      log_file = Merb::Config[:log_file]
+      raise "log file should be a string, got: #{log_file.inspect}" unless log_file.is_a?(String)
+      STDOUT.puts "Logging to file at #{log_file}" unless Merb.testing?
+      
+      # try to create log directory (if it doesnt exist)
+      log_directory = File.dirname(log_file)
+      FileUtils.mkdir_p(log_directory) unless File.exists?(log_directory)
+
+      Merb::Config[:log_stream] = File.open(log_file, "a")
     # but if it's not given, fallback to log stream or stdout
     else
       Merb::Config[:log_stream] ||= STDOUT
@@ -451,7 +456,7 @@ class Merb::BootLoader::Dependencies < Merb::BootLoader
   #
   # :api: private
   def self.set_encoding
-    unless Gem::Version.new(RUBY_VERSION) >= Gem::Version.new("1.9")
+    unless RUBY_VERSION >= '1.9'
       $KCODE = 'UTF8' if $KCODE == 'NONE' || $KCODE.blank?
     end
     
@@ -488,7 +493,8 @@ class Merb::BootLoader::Dependencies < Merb::BootLoader
     # :api: private
     def self.load_env_config
       if env_config?
-        STDOUT.puts "Loading #{env_config}" unless Merb.testing?
+        env_config_path = relative_to_merb_path(env_config)
+        STDOUT.puts "Loading #{env_config_path}" unless Merb.testing?
         load(env_config)
       end
       nil
@@ -518,7 +524,8 @@ class Merb::BootLoader::Dependencies < Merb::BootLoader
     def self.load_initfile
       return nil if Merb.const_defined?("INIT_RB_LOADED")
       if File.exists?(initfile)
-        STDOUT.puts "Loading init file from #{initfile}" unless Merb.testing?
+        initfile_path = relative_to_merb_path(initfile)
+        STDOUT.puts "Loading init file from #{initfile_path}" unless Merb.testing?
         load(initfile)
         Merb.const_set("INIT_RB_LOADED", true)
       elsif !Merb.testing?
@@ -549,6 +556,25 @@ class Merb::BootLoader::Dependencies < Merb::BootLoader
       @ran = true
       nil
     end
+
+    # Converts an absolute path to an path relative to Merbs root, if
+    # the path is in the Merb root dir. Otherwise it will return the
+    # absolute path.
+    #
+    # ==== Returns
+    # String:: Relative path or absolute
+    #
+    # :api: private
+    def self.relative_to_merb_path(path)
+      absolute_path = File.expand_path(path)
+      merb_root = File.expand_path(Merb.root)
+      if absolute_path.slice(0, merb_root.length) == merb_root
+        '.' + absolute_path.slice(merb_root.length..-1)
+      else
+        absolute_path
+      end
+    end
+
 end
 
 class Merb::BootLoader::MixinSession < Merb::BootLoader
@@ -691,6 +717,7 @@ class Merb::BootLoader::LoadClasses < Merb::BootLoader
       Merb.logger.warn! "Parent pid: #{Process.pid}"
       reader, writer = nil, nil
 
+      # Enable REE garbage collection
       if GC.respond_to?(:copy_on_write_friendly=)
         GC.copy_on_write_friendly = true
       end
@@ -742,14 +769,15 @@ class Merb::BootLoader::LoadClasses < Merb::BootLoader
             # We do not care about specific pid here.
             exit_status[1] && exit_status[1].exitstatus == 128 ? break : exit
           end
-          # wait for data to become available, timeout in 0.25 of a second
-          if select(reader_ary, nil, nil, 0.25)
+          # wait for data to become available, timeout in 0.5 of a second
+          if select(reader_ary, nil, nil, 0.5)
             begin
               # no open writers
               next if reader.eof?
               msg = reader.readline
-              if msg =~ /128/
-                Process.detach(pid)
+              reader.close
+              if msg.to_i == 128
+                Process.waitpid(pid, Process::WNOHANG)
                 break
               else
                 exit_gracefully
@@ -810,7 +838,10 @@ class Merb::BootLoader::LoadClasses < Merb::BootLoader
       Merb.exiting = true unless status == 128
 
       begin
-        @writer.puts(status.to_s) if @writer
+        if @writer
+          @writer.puts(status.to_s)
+          @writer.close
+        end
       rescue SystemCallError
       end
 
@@ -895,10 +926,12 @@ class Merb::BootLoader::LoadClasses < Merb::BootLoader
     def load_classes(*paths)
       orphaned_classes = []
       paths.flatten.each do |path|
-        Dir[path].each do |file|
+        Dir[path].sort.each do |file|
           begin
             load_file file
           rescue NameError => ne
+            Merb.logger.verbose! "Stashed file with missing requirements for later reloading: #{file}"
+            ne.backtrace.each_with_index { |line, idx| Merb.logger.verbose! "[#{idx}]: #{line}" }
             orphaned_classes.unshift(file)
           end
         end
@@ -1016,7 +1049,6 @@ class Merb::BootLoader::LoadClasses < Merb::BootLoader
         error_map = {}
 
         klasses.each do |klass|
-          klasses.delete(klass)
           begin
             load_file klass
           rescue NameError => ne
@@ -1024,6 +1056,7 @@ class Merb::BootLoader::LoadClasses < Merb::BootLoader
             failed_classes.push(klass)
           end
         end
+        klasses.clear
 
         # Keep list of classes unique
         failed_classes.each { |k| klasses.push(k) unless klasses.include?(k) }
@@ -1297,6 +1330,8 @@ class Merb::BootLoader::RackUpApplication < Merb::BootLoader
       Merb::Config[:app] = eval("::Rack::Builder.new {( #{rackup_code}\n )}.to_app", TOPLEVEL_BINDING, Merb::Config[:rackup])
     else
       Merb::Config[:app] = ::Rack::Builder.new {
+         use Merb::Rack::Head # handle head requests
+         use Merb::Rack::ContentLength # report content length
          if prefix = ::Merb::Config[:path_prefix]
            use Merb::Rack::PathPrefix, prefix
          end
@@ -1357,6 +1392,48 @@ class Merb::BootLoader::ReloadClasses < Merb::BootLoader
   def self.run
     return unless Merb::Config[:reload_classes]
 
+    TimedExecutor.every(Merb::Config[:reload_time] || 0.5) do
+      GC.start
+      reload!
+    end
+
+    nil
+  end
+
+  # Reloads all the files on the Merb application path
+  #
+  # ==== Returns
+  # nil
+  #
+  # :api: private
+  def self.reload!
+    reload(build_paths)
+  end
+
+  # Reloads all files which have been modified since they were last loaded.
+  #
+  # ==== Returns
+  # nil
+  #
+  # :api: private
+  def self.reload(paths = [])
+    paths.each do |file|
+      next if LoadClasses::MTIMES[file] &&
+        LoadClasses::MTIMES[file] == File.mtime(file)
+
+      LoadClasses.reload(file)
+    end
+
+    nil
+  end
+
+  # Returns a list of the paths on the merb application stack
+  #
+  # ==== Returns
+  # nil
+  #
+  # :api: private
+  def self.build_paths
     paths = []
     Merb.load_paths.each do |path_name, file_info|
       path, glob = file_info
@@ -1370,28 +1447,6 @@ class Merb::BootLoader::ReloadClasses < Merb::BootLoader
 
     paths.flatten!
 
-    TimedExecutor.every(Merb::Config[:reload_time] || 0.5) do
-      GC.start
-      reload(paths)
-    end
-
-    nil
-  end
-
-  # Reloads all files which have been modified since they were last loaded.
-  #
-  # ==== Returns
-  # nil
-  #
-  # :api: private
-  def self.reload(paths)
-    paths.each do |file|
-      next if LoadClasses::MTIMES[file] &&
-        LoadClasses::MTIMES[file] == File.mtime(file)
-
-      LoadClasses.reload(file)
-    end
-
-    nil
+    return paths
   end
 end
