@@ -5,15 +5,20 @@ describe Merb::BootLoader::RackUpApplication do
   it "should default to rack config (rack.rb)" do
     options = {:merb_root => File.dirname(__FILE__) / 'fixture'}
     Merb::Config.setup(options)
-    app = Merb::BootLoader::RackUpApplication.run
-    app.class.should == Merb::Rack::Application
+    Merb::BootLoader::default_framework
+    Merb::BootLoader::RackUpApplication.run
+    app = Merb::Config[:app]
+    app.class.should == Merb::Rack::Static
   end
 
   it "should use rackup config that we specified" do
     options = {:rackup => File.dirname(__FILE__) / 'fixture' / 'config' / 'black_hole.rb'}
     Merb::Config.setup(options)
-    app = Merb::BootLoader::RackUpApplication.run
-    app.class.should == Rack::Adapter::BlackHole
+    Merb::BootLoader::RackUpApplication.run
+    app = Merb::Config[:app]
+
+    # 1.9 returns "#<Class:0xa716858>::Rack::Adapter::BlackHole"
+    app.class.name.should include("Rack::Adapter::BlackHole")
 
     env = Rack::MockRequest.env_for("/black_hole")
     status, header, body = app.call(env)
@@ -21,4 +26,5 @@ describe Merb::BootLoader::RackUpApplication do
     header.should == { "Content-Type" => "text/plain" }
     body.should == ""
   end
+  
 end
