@@ -3,9 +3,10 @@
 # However, Merb::BootLoader.subclasses is not considered public API and should not be used in plugins.
 
 require File.join(File.dirname(__FILE__), "spec_helper")
+startup_merb
 
 class Merb::BootLoader::AfterTest < Merb::BootLoader
-  after Merb::BootLoader::BeforeAppRuns
+  after Merb::BootLoader::BeforeAppLoads
   
   def self.run
   end
@@ -21,7 +22,7 @@ end
 describe "The BootLoader" do
   
   it "should support adding a BootLoader after another" do
-    idx = Merb::BootLoader.subclasses.index("Merb::BootLoader::BeforeAppRuns")
+    idx = Merb::BootLoader.subclasses.index("Merb::BootLoader::BeforeAppLoads")
     Merb::BootLoader.subclasses.index("Merb::BootLoader::AfterTest").should == idx + 1
   end
 
@@ -30,4 +31,11 @@ describe "The BootLoader" do
     Merb::BootLoader.subclasses.index("Merb::BootLoader::BeforeTest").should == idx - 1
   end
   
-end
\ No newline at end of file
+  describe Merb::BootLoader::ChooseAdapter do
+    it "should check if we're running irb and if so set irb as adapter" do
+      Merb.stub!(:running_irb?).and_return(true)
+      Merb::BootLoader::ChooseAdapter.run
+      Merb::Config[:adapter].should eql 'irb'
+    end
+  end
+end
