@@ -1,3 +1,4 @@
+require 'json'
 class SpecHelperController < Merb::Controller
   
   def index
@@ -18,10 +19,12 @@ class SpecHelperController < Merb::Controller
   
   def create
     Merb::Test::ControllerAssertionMock.called(:create)
+    params.to_json
   end
   
   def update
     Merb::Test::ControllerAssertionMock.called(:update)
+    params.to_json
   end
   
   def destroy
@@ -35,4 +38,5 @@ module Namespaced
       Merb::Test::ControllerAssertionMock.called(:index)
     end
   end
-end
\ No newline at end of file
+end  
+        
